@@ -57,6 +57,7 @@
 static GMainLoop *main_loop;
 static GDBusProxy *index_proxy;
 static GPtrArray *proxy_folders;
+static guint cleanup_id;
 
 static gint initial_sleep = -1;
 static gboolean no_daemon;
@@ -432,6 +433,37 @@ miner_start (TrackerMiner  *miner,
 	                                           miner);
 }
 
+static gboolean
+cleanup_cb (gpointer user_data)
+{
+	/* Reclaim as much memory as possible */
+	malloc_trim (0);
+
+	cleanup_id = 0;
+
+	return G_SOURCE_REMOVE;
+}
+
+#if GLIB_CHECK_VERSION (2, 64, 0)
+static void
+on_low_memory (GMemoryMonitor            *monitor,
+               GMemoryMonitorWarningLevel level,
+               gpointer                   user_data)
+{
+	if (level > G_MEMORY_MONITOR_WARNING_LEVEL_LOW)
+		malloc_trim (0);
+}
+#endif
+
+static void
+miner_started_cb (TrackerMinerFS *fs)
+{
+	if (cleanup_id) {
+		g_source_remove (cleanup_id);
+		cleanup_id = 0;
+	}
+}
+
 static void
 miner_finished_cb (TrackerMinerFS *fs,
                    gdouble         seconds_elapsed,
@@ -451,6 +483,8 @@ miner_finished_cb (TrackerMinerFS *fs,
 							 TRUE);
 	}
 
+	cleanup_id = g_timeout_add_seconds (30, cleanup_cb, NULL);
+
 	/* We're not sticking around for file updates, so stop
 	 * the mainloop and exit.
 	 */
@@ -907,6 +941,9 @@ main (gint argc, gchar *argv[])
 	TrackerEndpointDBus *endpoint;
 	TrackerDomainOntology *domain_ontology;
 	GCancellable *cancellable;
+#if GLIB_CHECK_VERSION (2, 64, 0)
+	GMemoryMonitor *memory_monitor;
+#endif
 	gchar *domain_name, *dbus_name;
 
 	main_loop = NULL;
@@ -974,25 +1011,24 @@ main (gint argc, gchar *argv[])
 
 	main_loop = g_main_loop_new (NULL, FALSE);
 
-	if (domain_ontology && domain_ontology_name) {
-		/* If we are running for a specific domain, we tie the lifetime of this
-		 * process to the domain. For example, if the domain name is
-		 * org.example.MyApp then this tracker-miner-fs process will exit as
-		 * soon as org.example.MyApp exits.
-		 */
+	if (no_daemon) {
+		g_message ("tracker-miner-fs-3 running in --no-daemon mode.");
+	} else if (domain_ontology && domain_ontology_name) {
 		domain_name = tracker_domain_ontology_get_domain (domain_ontology, NULL);
+
+		g_message ("tracker-miner-fs-3 running in --domain-ontology mode as "
+		           "%s." DBUS_NAME_SUFFIX ". The service will exit when %s "
+		           "disappears from the bus.", domain_name, domain_name);
+
 		g_bus_watch_name_on_connection (connection, domain_name,
 		                                G_BUS_NAME_WATCHER_FLAGS_NONE,
 		                                NULL, on_domain_vanished,
 		                                main_loop, NULL);
 		g_free (domain_name);
+	} else {
+		g_message ("tracker-miner-fs-3 running as org.freedesktop." DBUS_NAME_SUFFIX);
 	}
 
-	TRACKER_NOTE (CONFIG, g_message ("Checking if we're running as a daemon:"));
-	TRACKER_NOTE (CONFIG, g_message ("  %s %s",
-	                      no_daemon ? "No" : "Yes",
-	                      no_daemon ? "(forced by command line)" : ""));
-
 	if (!dry_run) {
 		GFile *store = get_cache_dir (domain_ontology);
 		tracker_error_report_init (store);
@@ -1093,10 +1129,18 @@ main (gint argc, gchar *argv[])
 		tracker_miner_files_set_mtime_checking (TRACKER_MINER_FILES (miner_files), do_mtime_checking);
 	}
 
+	g_signal_connect (miner_files, "started",
+			  G_CALLBACK (miner_started_cb),
+			  NULL);
 	g_signal_connect (miner_files, "finished",
 			  G_CALLBACK (miner_finished_cb),
 			  NULL);
 
+#if GLIB_CHECK_VERSION (2, 64, 0)
+	memory_monitor = g_memory_monitor_dup_default ();
+	g_signal_connect (memory_monitor, "low-memory-warning", on_low_memory, NULL);
+#endif
+
 	/* Preempt creation of graphs */
 	tracker_sparql_connection_update_async (tracker_miner_get_connection (miner_files),
 	                                        "CREATE SILENT GRAPH tracker:FileSystem; "
@@ -1139,6 +1183,11 @@ main (gint argc, gchar *argv[])
 	tracker_sparql_connection_close (sparql_conn);
 	g_object_unref (sparql_conn);
 
+#if GLIB_CHECK_VERSION (2, 64, 0)
+	g_signal_handlers_disconnect_by_func (memory_monitor, on_low_memory, NULL);
+	g_object_unref (memory_monitor);
+#endif
+
 	g_print ("\nOK\n\n");
 
 	return EXIT_SUCCESS;
